from spaceone.core.error import *


class ERROR_INVALID_REPOSITORY_TYPE(ERROR_INVALID_ARGUMENT):
    _message = "Invalid repository type. (supported_types = local | remote | managed)"


class ERROR_NOT_UPDATE_MANAGED_REPOSITORY(ERROR_INVALID_ARGUMENT):
    _message = "Can not update managed repository."


class ERROR_LOCAL_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Local repository already exist."


class ERROR_ONLY_ONE_LOCAL_REPOSITORY_CAN_BE_REGISTERED(ERROR_INVALID_ARGUMENT):
    _message = "Only one local repository can be registered.(local_repository_count= {local_repository_count})"


class ERROR_ONLY_ONE_MANAGED_REPOSITORY_CAN_BE_REGISTERED(ERROR_INVALID_ARGUMENT):
    _message = "Only one managed repository can be registered.(managed_repository_count= {managed_repository_count})"


class ERROR_MANAGED_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Managed repository already exist."


class ERROR_REMOTE_REPOSITORY_ALREADY_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Remote repository already exist. (endpoint = {endpoint})"


class ERRROR_NOT_SET_UP_REMOTE_REPOSITORY(ERROR_INVALID_ARGUMENT):
    _message = "Remote repository is not set up."


class ERROR_SORT_KEY(ERROR_INVALID_ARGUMENT):
    _message = "Sort key is invalid. (sort_key = {sort_key})"


class ERROR_NOT_FOUND_REPOSITORY(ERROR_INVALID_ARGUMENT):
    _message = "Repository does not exist. (repository_id = {repository_id})"


class ERROR_REMOTE_REPOSITORY_AUTH_FAILURE(ERROR_INVALID_ARGUMENT):
    _message = "Remote repository authentication failure."



import logging

from spaceone.core import config
from spaceone.core.connector.space_connector import SpaceConnector

from spaceone.repository.error import *
from spaceone.repository.manager.plugin_manager import PluginManager
from spaceone.repository.manager.repository_manager import RepositoryManager

__all__ = ["RemotePluginManager"]

_LOGGER = logging.getLogger(__name__)


class RemotePluginManager(PluginManager):
    def get_plugin(self, repo_info: dict, plugin_id: str, domain_id: str = None):
        endpoint = repo_info["endpoint"]
        market_place_token = repo_info["token"]

        remote_repo_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", endpoint=endpoint, token=market_place_token
        )

        plugin_info: dict = remote_repo_conn.dispatch(
            "Plugin.get", {"plugin_id": plugin_id, "repository_id": "repo-local"}
        )

        return self.change_response(plugin_info, repo_info)

    def list_plugins(self, repo_info: dict, query: dict, params: dict):
        try:
            endpoint = repo_info["endpoint"]
            market_place_token = repo_info["token"]

            _LOGGER.debug(f"[list_plugins] Remote Repository endpoint: {endpoint}")

            remote_repo_conn: SpaceConnector = self.locator.get_connector(
                SpaceConnector, endpoint=endpoint, token=market_place_token
            )

            response = remote_repo_conn.dispatch(
                "Plugin.list", {"query": query, "repository_id": "repo-local"}
            )

            plugins_info = response.get("results", [])
            total_count = response.get("total_count", 0)
            results = []

            for plugin_info in plugins_info:
                results.append(self.change_response(plugin_info, repo_info))

            return results, total_count
        except Exception as e:
            _LOGGER.error(f"[list_plugins] {e}")
            return [], 0

    def get_plugin_versions(self, repo_info: dict, plugin_id: str, domain_id: str):
        endpoint = repo_info["endpoint"]
        market_place_token = repo_info["token"]

        remote_repo_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", endpoint=endpoint, token=market_place_token
        )
        response = remote_repo_conn.dispatch(
            "Plugin.get_versions",
            {"plugin_id": plugin_id, "repository_id": "repo-local"},
        )

        versions = response.get("results", [])
        total_count = response.get("total_count", 0)

        _LOGGER.debug(f"[get_plugin_version] total version count: {total_count}")

        return versions



from schematics.models import Model
from schematics.types import ListType, StringType, BooleanType


class Capability(Model):
    supported_schema = ListType(StringType())
    use_resource_secret = BooleanType(default=False)
    monitoring_type = StringType(choices=('METRIC', 'LOG'))
    supported_providers = ListType(StringType())



import logging
import os
import copy
import pandas as pd
import re

from spaceone.core import utils, config
from spaceone.repository.error import *
from spaceone.repository.manager.plugin_manager import PluginManager
from spaceone.repository.manager.repository_manager import RepositoryManager

__all__ = ["ManagedPluginManager"]

_LOGGER = logging.getLogger(__name__)
_REGISTRY_CONNECTOR_MAP = {
    "DOCKER_HUB": "DockerHubConnector",
    "AWS_PRIVATE_ECR": "AWSPrivateECRConnector",
    "HARBOR": "HarborConnector",
}
_BASE_DIR = os.path.join(os.path.dirname(__file__), "../../managed_resource/plugin/")
_MANAGED_PLUGINS = []

for filename in os.listdir(_BASE_DIR):
    if filename.endswith(".yaml"):
        file_path = os.path.join(_BASE_DIR, filename)
        plugin_info = utils.load_yaml_from_file(file_path)
        _MANAGED_PLUGINS.append(plugin_info)


class ManagedPluginManager(PluginManager):
    def __init__(self):
        super().__init__()
        self.managed_registry_type = config.get_global(
            "MANAGED_REGISTRY_TYPE", "DOCKER_HUB"
        )
        self.managed_registry_config = config.get_global("MANAGED_REGISTRY_CONFIG", {})
        self.managed_plugin_image_prefix = config.get_global(
            "MANAGED_PLUGIN_IMAGE_PREFIX", "cloudforet"
        )
        self.managed_plugin_df = self._load_managed_plugins()

    def get_plugin(self, repo_info: dict, plugin_id: str, domain_id: str):
        managed_plugin_df = self._filter_managed_plugins(plugin_id)

        if len(managed_plugin_df) == 0:
            raise ERROR_NOT_FOUND(key="plugin_id", value=plugin_id)

        managed_plugins_info = managed_plugin_df.to_dict("records")

        return self.change_managed_plugin_info(
            managed_plugins_info[0], repo_info, domain_id
        )

    def list_plugins(self, repo_info: dict, query: dict, params: dict):
        plugin_id = params.get("plugin_id")
        name = params.get("name")
        resource_type = params.get("resource_type")
        provider = params.get("provider")
        domain_id = params.get("domain_id")
        sort = query.get("sort", [])
        page = query.get("page", {})
        keyword = query.get("keyword")

        managed_plugin_df = self._filter_managed_plugins(
            plugin_id, name, resource_type, provider, keyword
        )
        managed_plugin_df = self._sort_managed_plugins(managed_plugin_df, sort)
        total_count = len(managed_plugin_df)
        managed_plugin_df = self._page_managed_plugins(managed_plugin_df, page)

        results = []
        for managed_plugin_info in managed_plugin_df.to_dict("records"):
            results.append(
                self.change_managed_plugin_info(
                    managed_plugin_info, repo_info, domain_id
                )
            )

        return results, total_count

    def get_plugin_versions(self, repo_info: dict, plugin_id: str, domain_id: str):
        managed_plugin_info = self.get_plugin(repo_info, plugin_id, domain_id)
        registry_type = managed_plugin_info["registry_type"]
        image = managed_plugin_info["image"]

        registry_url = (
            config.get_global("REGISTRY_INFO", {}).get(registry_type).get("url")
        )
        registry_connector = _REGISTRY_CONNECTOR_MAP[registry_type]

        try:
            connector = self.locator.get_connector(registry_connector)
            tags = connector.get_tags(registry_url, image)
        except Exception as e:
            _LOGGER.error(f"[get_plugin_versions] get_tags error: {e}", exc_info=True)
            raise e
        else:
            return tags

    @staticmethod
    def _load_managed_plugins():
        managed_plugin_df = pd.DataFrame(copy.deepcopy(_MANAGED_PLUGINS))
        return managed_plugin_df.fillna("")

    def _filter_managed_plugins(
        self, plugin_id=None, name=None, resource_type=None, provider=None, keyword=None
    ):
        managed_plugin_df = copy.deepcopy(self.managed_plugin_df)

        if plugin_id:
            managed_plugin_df = managed_plugin_df[
                managed_plugin_df["plugin_id"] == plugin_id
            ]

        if name:
            managed_plugin_df = managed_plugin_df[managed_plugin_df["name"] == name]

        if resource_type:
            managed_plugin_df = managed_plugin_df[
                managed_plugin_df["resource_type"] == resource_type
            ]

        if provider:
            managed_plugin_df = managed_plugin_df[
                managed_plugin_df["provider"] == provider
            ]

        if keyword:
            managed_plugin_df = managed_plugin_df[
                managed_plugin_df["name"].str.contains(keyword, flags=re.IGNORECASE)
            ]

        return managed_plugin_df

    @staticmethod
    def _sort_managed_plugins(managed_plugin_df: pd.DataFrame, sort: list):
        for sort_condition in sort:
            if sort_key := sort_condition.get("key"):
                desc = sort_condition.get("desc", False)
                try:
                    return managed_plugin_df.sort_values(
                        by=sort_key, ascending=not desc
                    )
                except Exception as e:
                    raise ERROR_SORT_KEY(sort_key=sort_key)
        else:
            return managed_plugin_df

    @staticmethod
    def _page_managed_plugins(managed_plugin_df: pd.DataFrame, page: dict):
        if limit := page.get("limit"):
            start = page.get("start", 1) - 1
            return managed_plugin_df[start : start + limit]
        else:
            return managed_plugin_df

    def change_managed_plugin_info(self, plugin_info: dict, repo_info: dict, domain_id):
        plugin_info["state"] = "ENABLED"
        plugin_info["registry_type"] = self.managed_registry_type
        plugin_info["registry_config"] = self.managed_registry_config
        plugin_info[
            "image"
        ] = f"{self.managed_plugin_image_prefix}/{plugin_info['image']}"

        if repo_info:
            plugin_info["repository_info"] = {
                "repository_id": repo_info["repository_id"],
                "name": repo_info["name"],
                "repository_type": repo_info["repository_type"],
            }

        if domain_id:
            plugin_info["domain_id"] = domain_id

        return plugin_info



from spaceone.api.repository.v1 import repository_pb2, repository_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Repository(BaseAPI, repository_pb2_grpc.RepositoryServicer):
    pb2 = repository_pb2
    pb2_grpc = repository_pb2_grpc

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        with self.locator.get_service("RepositoryService", metadata) as repo_svc:
            repos_info = repo_svc.list(params)
            return self.locator.get_info(
                "RepositoriesInfo",
                repos_info,
                minimal=self.get_minimal(params),
            )
